( cd usr/lib ; rm -rf libgicaction.so )
( cd usr/lib ; ln -sf libgicaction.so.0.0.1 libgicaction.so )
( cd usr/lib ; rm -rf libgiigic.so.1 )
( cd usr/lib ; ln -sf libgiigic.so.1.0.1 libgiigic.so.1 )
( cd usr/lib ; rm -rf libgiigic.so )
( cd usr/lib ; ln -sf libgiigic.so.1.0.1 libgiigic.so )
( cd usr/lib ; rm -rf libgicaction.so.0 )
( cd usr/lib ; ln -sf libgicaction.so.0.0.1 libgicaction.so.0 )
