/* $Id: gicaction_lazy.h,v 1.4 2004/11/28 00:15:37 soyt Exp $
 * lazy.h - (c) 1999 Andreas Beck   becka@ggi-project.org
 *
 *   This software is placed in the public domain and can be used freely
 *   for any purpose. It comes without any kind of warranty, either
 *   expressed or implied, including, but not limited to the implied
 *   warranties of merchantability or fitness for a particular purpose.
 *   Use it at your own risk. the author is not responsible for any damage
 *   or consequences raised by use or inability to use this program.
 */

#ifndef _GGI_GICACTION_LAZY_H
#define _GGI_GICACTION_LAZY_H

#include <ggi/gic.h>
#include <ggi/gic-defs.h>

#define MAXRECOGNIZERS 32

typedef struct gicActionLazyData {

	gic_state	current;
	gic_state	pulse;
	
	gic_state	array[MAXRECOGNIZERS];
} gicActionLazyData;

GGIGICACTIONAPIFUNC void gicActionLazyAction(
	gic_handle_t hand,gic_actionlist *action,gic_feature *feature,
	gic_state newstate,gic_flag flag,int recnum);
GGIGICACTIONAPIFUNC gic_state gicActionLazyGetstate(gicActionLazyData *data);
GGIGICACTIONAPIFUNC void gicActionLazyReset(gicActionLazyData *data);

#endif /* _GGI_GICACTION_LAZY_H */
